package com.chelvc.cloud.maintain.controller;

import com.chelvc.cloud.maintain.copier.MerchantCopier;
import com.chelvc.cloud.maintain.vo.SimpleMerchantVO;
import com.chelvc.cloud.vehicle.api.dto.BalanceDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantBalanceDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.api.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.api.param.MerchantQueryParam;
import com.chelvc.cloud.vehicle.api.param.MerchantRankParam;
import com.chelvc.cloud.vehicle.api.param.OmsCartItemPagingParam;
import com.chelvc.cloud.vehicle.api.param.QueryMerchantBalanceParam;
import com.chelvc.cloud.vehicle.api.service.FavoriteService;
import com.chelvc.cloud.vehicle.api.service.BalanceDetailService;
import com.chelvc.cloud.vehicle.api.service.MerchantService;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * 商家接口
 *
 * @author Woody
 * @date 2023/7/20
 */
@Validated
@RestController
@ResponseWrapping
public class MerchantController {
    @DubboReference
    private MerchantService merchantService;

    @DubboReference
    private FavoriteService favoriteService;

    @DubboReference
    private BalanceDetailService balanceDetailService;

    /**
     * 获取商家列表
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchantSort")
    public List<SimpleMerchantVO> listSimpleMerchants(@Valid MerchantQueryParam param) {
        return MerchantCopier.INSTANCE.copying(this.merchantService.listSimpleMerchants(param));
    }

    /**
     * 根据地区获取商家排名
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchants/rank")
    public List<SimpleMerchantVO> listRankMerchants(@Valid MerchantRankParam param) {
        return MerchantCopier.INSTANCE.copying(this.merchantService.listRankMerchants(param, SessionContextHolder.getId()));
    }

    /**
     * 获取商家详情
     *
     * @param id 商家ID
     * @param param 查询参数
     * @return 商家详情
     */
    @GetMapping("/merchant/{id}")
    public MerchantDetailDTO getMerchantDetail(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id,
                                               @Valid LocationQueryParam param) {
        return this.merchantService.getMerchantDetail(id, param);
    }

    /**
     * 收藏商家信息
     *
     * @param id 商家ID
     * @return 收藏ID
     */
    @PostMapping("/merchant/{id}/favorite")
    public Long addMerchantFavorite(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id) {
        return this.favoriteService.addMerchantFavorite(id,"MERCHANT");
    }

    /**
     * 查询商家余额明细
     *
     * @param param 查询条件
     * @param pageNum
     * @param pageSize
     * @return 收藏ID
     */
    @GetMapping("/merchant/balance/detail")
    public Pagination<BalanceDetailDTO> queryBalanceDetail(@Validated QueryMerchantBalanceParam param, Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        return this.balanceDetailService.queryByUserId(param, pageNum, pageSize, SessionContextHolder.getId(), 2);
    }


    /**
     * 我的购物车列表
     *
     */
    @GetMapping("/mineCartList")
    public Pagination<MerchantDTO> mineCartList(@Valid OmsCartItemPagingParam param) {
        return this.merchantService.getUserMineCartPaging(param);
    }

}
